# Page Components

Root level Components that are rendered directly by the Router Switch Component

## What should go in here:

- Components that are rendered directly by Routers root level Switch Component

## What shouldn't go in here:

- Things like modals that are controlled by the route but not controlled by the root level Switch Component

## Hijacking this

### The docker containers

- The docker container running the web application `kitty-items_web` is on `localhost:3001`, so go here to see the web app.

**New interface**:
- The docker container running the web application `kitty-items_web` is on `localhost:3001` no longer works because of a comment made in `index.js`, go to `localhost:3001/newInterface` to see the new interface.

- The docker container running the wallets and accounts `ghcr.io/onflow/fcl-dev-wallet` is on `localhost:8701`.

- The docker container running the kitty-items api `kitty-items_api` is on `localhost:3000`.

- The docker container running the database `postgres:13-alpine ` is on `localhost:5432` (**this port is not public exposed, going to this link will yield nothing**), to access this data base follow this steps:

    - To go inside the database go with: `docker exec -it kitty-items_db_1 bash`.
    - Then run `psql kittyitems -U kittyuser` this parameters are in the `.env.dev` file.
    - To see the names of the created tables run: 
    ```sql
    SELECT table_name FROM information_schema.tables WHERE table_schema='public'
    ```
- The docker container running the flow registery `gcr.io/flow-container-registry/emulator` is on port `localhost:3569`  (**this port is not public exposed, going to this link will yield nothing**).

### The react app

After login, the react that we see is `./pages/account/index.js`. Which runs with `./parts/account-items-cluster.comp.js` All of this runs with the component library https://chakra-ui.com/

### Antd

To run **Ant Design** I did:
1) `npm install antd` outside of the container in the web folder.
2) I deleted the previous containers with `docker rm [containers-id]`, and executed this commands again:

    2.1 Run `npm install` in the root of the project.

    2.2 Run `npx lerna exec npm install` to install project dependencies.
3) Run `npm run start:dev` 
4) Run `flow project deploy --network emulator`

If this isn't enough to make it work, I also did:

1) `docker exec -it [ontainer id of the web container] sh`
2) `npm install antd`
3) Exit the container with `exit`.


**To see containers id do this command: `docker container ls`
### SASS

To run **SASS** and *.scss* files I did:
1) `npm install sass` *outside* of the container in the web folder.

2) Run `npm install` in the root of the project.

2) Run `npx lerna exec npm install` to install project dependencies.

3) Run `npm run start:dev` 
4) Run `flow project deploy --network emulator`

If this isn't enough to make it work, I also did:

1) `docker exec -it [ontainer id of the web container] sh`
2) `npm install sass`
3) Exit the container with `exit`.

This is installing sass *inside* the container

I also did:
1) `docker restart [container id of the web container]`
2) Reload the `localhost:3001`

### Three.js

To run **Three.js** I did:
1) `npm install --save three` outside of the container in the web folder.

2) Run `npm install` in the root of the project.

3) Run `npx lerna exec npm install` to install project dependencies.

I also installed **Three.js** *inside* the container:

1) `docker exec -it [container id of the web container] sh`
2) `npm install --save three`
3) Exit the container with `exit`.
4) `docker restart [container id of the web container]`


### Model Viewer
To run `@google/model-viewer` I did:
1) `npm install --save @google/model-viewer` outside of the container in the web folder.
2) Run `npm install` in the root of the project.
3) Run `npx lerna exec npm install` to install project dependencies.

I also installed **@google/model-viewer** *inside* the container:

1) `docker exec -it [container id of the web container] sh`
2) `npm install --save @google/model-viewer`
3) Exit the container with `exit`.
4) `docker restart [container id of the web container]`

### react-uuid
To run `react-uuid` I did:
1) `npm i react-uuid` outside of the container in the web folder.
2) Run `npm install` in the root of the project.
3) Run `npx lerna exec npm install` to install project dependencies.

I also installed **react-uuid** *inside* the container:

1) `docker exec -it [container id of the web container] sh`
2) `npm i react-uuid`
3) Exit the container with `exit`.
4) `docker restart [container id of the web container]`


### How to add a new drop:
1) Add the `.json` in `./data/models/drops` and import to `registery.js` following the convetion.
2) In `../index.js` import the drop and add this lines inside the `<Switch>` component:
```js
{group_by_cat(drop3).map((category) =>
            <Route exact path={"/assets"+"/drop3/"+category} render={(props) => <NewInterface />}/>
            )}
```
3) In `./components/ParcelList.jsx` add 
```jsx
case "drop3":
            return(
            <Row justify> 
            {drop3.filter(models_category => models_category["category_name"]===categoryType).map((filteredmodel) =>
                <Col className="model_render">
                    <ModelRender path_url={filteredmodel["route"]} path_preview={filteredmodel["preview"]} description={filteredmodel["description"]}/>
                </Col>
            )
            }
            </Row>
            );
```
inside the `switch` and also import the drop from `registery.js`

4) And lastly in the `./components/MenuP.jsx` add:

```js
<SubMenu key="drop3" title="DROP 3" style={{whiteSpace: 'normal', height: 'auto'}}>
                    {group_by_cat(drop3).map((category) =>
                    <Menu.Item key={"/assets"+"/drop3/"+category}>
                        {category}
                        <Link to={"/assets"+"/drop3/"+category}></Link>
                    </Menu.Item>
                    )}
                </SubMenu>
```
inside the `<Menu>` componenent.
**This example works witht he example of a new drop called `drop3.json` and imported as `drop3`.
